package xyz.tbvns.core;

import org.apache.commons.io.FileUtils;
import xyz.tbvns.core.Objects.Level;

import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Base64;
import java.util.List;

public class DataManager {
    public static List<String> decodeLocalLevel() throws IOException {
        List<String> data = new ArrayList<>();
        File geometryDash = new File(FilesLocations.getGeometryDashFolder());
        String[] files = geometryDash.list();
        for (String file : files) {
            if (file.contains("CCLocalLevels") && file.endsWith(".dat")) {
                File fileToDecode = new File(geometryDash.getPath() + "/" + file);
                data.add(new String(Decoder.decodeFile(fileToDecode, 11)));
            }
        }
        return data;
    }

    private static String rawData;
    public static List<Level> getLevels() throws IOException {
        List<Level> levelList = new ArrayList<>();
        rawData = XmlUtils.beautify(decodeLocalLevel().get(0));
        String[] lineData = rawData.split("\n");
        Level level = new Level();
        int endTime = 0;
        for (int i = 0; i < lineData.length; i++) {
            String line = lineData[i];
            if (line.startsWith("<k>k_") && line.endsWith("</k>")) {
                level = new Level();
            } else
            if (line.equals("<k>k2</k>")) {
                level.setName(lineData[i+1].replace("<s>", "").replace("</s>", ""));
                level.setNameLine(i+1);
            } else
            if (line.equals("<k>k3</k>")) {
                level.setDescription(new String(Base64.getUrlDecoder().decode(lineData[i+1].replace("<s>", "").replace("</s>", ""))));
                level.setDescriptionLine(i+1);
            } else
            if (line.equals("<k>k4</k>")) {
                level.setEncodedLevelString(lineData[i+1].replace("<s>", "").replace("</s>", ""));
                level.setEncodedLevelStringLine(i+1);
            } else
            if (line.equals("<k>k5</k>")) {
                level.setCreator(lineData[i+1].replace("<s>", "").replace("</s>", ""));
                level.setCreatorLine(i+1);
            } else
            if (line.equals("</d>")) {
                endTime++;
                if (endTime % 2 == 0) {
                    levelList.add(level);
                }
            } else
            if (line.equals("<k>LLM_02</k>")) {
                break;
            }
        }
        return levelList;
    }

    public static void saveLevel(Level level) throws IOException {
        String[] data = rawData.split("\n");
        System.out.println(level.getEncodedLevelStringLine() + " " + level.getCreatorLine() + " " + level.getNameLine() + " " + level.getDescriptionLine());
        data[level.getEncodedLevelStringLine()] = "<s>" + level.getEncodedLevelString() + "</s>";
        data[level.getNameLine()] = "<s>" + level.getName() + "</s>";
        data[level.getCreatorLine()] = "<s>" + level.getCreator() + "</s>";
        if (level.getDescriptionLine() != 0) {
            data[level.getDescriptionLine()] = "<s>" + new String(Base64.getUrlEncoder().encode(level.getDescription().getBytes())) + "</s>";
        }

        StringBuilder decodedData = new StringBuilder();
        for (String s : data) {
            decodedData.append(s);
        }
        File file = new File(FilesLocations.getGeometryDashFolder() + "/CCLocalLevels.dat");
        FileUtils.writeStringToFile(file, decodedData.toString().replace("\n", ""), StandardCharsets.UTF_8, false);
        Decoder.encodeFile(file, 11);
    }

    public static void saveLevel(Level level, UIProgressBar progressBar, float factor) throws IOException {
        progressBar.updateBar(progressBar.getProgressBar().getProgress() + 0.2 * factor);
        String[] data = rawData.split("\n");
        data[level.getEncodedLevelStringLine()] = "<s>" + Decoder.encodeLevel(level.getEncodedLevelString()) + "</s>";
        data[level.getNameLine()] = "<s>" + level.getName() + "</s>";
        data[level.getCreatorLine()] = "<s>" + level.getCreator() + "</s>";
        if (level.getDescriptionLine() != 0) {
            data[level.getDescriptionLine()] = "<s>" + new String(Base64.getUrlEncoder().encode(level.getDescription().getBytes())) + "</s>";
        }
        progressBar.updateBar(progressBar.getProgressBar().getProgress() + 0.2 * factor);

        StringBuilder decodedData = new StringBuilder();
        for (String s : data) {
            decodedData.append(s);
        }
        progressBar.updateBar(progressBar.getProgressBar().getProgress() + 0.2 * factor);

        File file = new File(FilesLocations.getGeometryDashFolder() + "/CCLocalLevels.dat");
        FileUtils.writeStringToFile(file, decodedData.toString().replace("\n", ""), StandardCharsets.UTF_8, false);
        progressBar.updateBar(progressBar.getProgressBar().getProgress() + 0.2 * factor);
        Decoder.encodeFile(file, 11);
    }
}
