package xyz.tbvns.ui.LevelEditor.tabs;

import javafx.scene.Node;
import javafx.scene.control.*;
import javafx.scene.layout.FlowPane;
import javafx.scene.layout.Pane;
import javafx.stage.FileChooser;
import oshi.util.Util;
import xyz.tbvns.core.DataManager;
import xyz.tbvns.core.Decoder;
import xyz.tbvns.core.Objects.Level;
import xyz.tbvns.ui.LevelEditor.FXTab;
import xyz.tbvns.core.UIProgressBar;
import xyz.tbvns.ui.LevelEditor.SaveManager;
import xyz.tbvns.ui.Utils;

import java.io.File;
import java.io.IOException;
import java.net.URISyntaxException;

public class EditLevelString implements FXTab {
    private FlowPane pane;
    private TextArea area;
    private ToolBar toolBar;

    @Override
    public Pane getPane(Level level) {
        pane = new FlowPane();
        String levelString = "";
        try {
            levelString = Decoder.decodeLevel(level.getEncodedLevelString()).replace(";", ";\n").replace("_", ",");
        } catch (Exception e) {
            throw new RuntimeException(e);
        }

        //Replace this with https://github.com/FXMisc/RichTextFX
        area = new TextArea(levelString);
        pane.getChildren().add(area);

        area.setOnKeyTyped(keyEvent -> {
            level.setEncodedLevelString(area.getText().replace("\n", ""));
        });

        Button save = new Button("Save");
        save.setOnMouseClicked(mouseEvent -> {
            SaveManager.save(level);
        });
        Button export = new Button("Export");
        export.setOnMouseClicked(mouseEvent -> {
            SaveManager.export(pane, level);
        });
        toolBar = new ToolBar(save, export);
        pane.getChildren().add(toolBar);

        return pane;
    }

    @Override
    public String getTitle() {
        return "Level string";
    }

    @Override
    public void setupResize(TabPane parent) {
        parent.widthProperty().addListener((observableValue, number, t1) -> {
            toolBar.setPrefWidth(t1.floatValue());
            area.setPrefWidth(t1.floatValue());
        });

        parent.heightProperty().addListener((observableValue, number, t1) -> {
            area.setPrefHeight(t1.floatValue() - toolBar.getHeight() - parent.getTabMaxHeight() - 7);
        });
    }
}
