package xyz.tbvns.ui;

import javafx.application.Platform;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.Node;
import javafx.scene.control.Label;
import javafx.scene.control.ScrollPane;
import javafx.scene.layout.*;
import javafx.scene.paint.Color;
import javafx.scene.text.Font;
import javafx.stage.Stage;
import lombok.extern.slf4j.Slf4j;
import xyz.tbvns.core.DataManager;
import xyz.tbvns.core.Objects.Level;
import xyz.tbvns.core.UIProgressBar;
import xyz.tbvns.ui.LevelEditor.MainWindow;

import java.util.List;

@Slf4j
public class LevelSelectScreen {

    public void open() {

    }

    public static Label label = new Label("Select the level to edit:");
    public static Pane pane() {
        FlowPane pane = new FlowPane();
        pane.setAlignment(Pos.TOP_CENTER);
        pane.setPrefWidth(400);
        pane.setMinWidth(400);
        pane.setMinHeight(400);
        pane.setMaxHeight(500);

        label.setPadding(new Insets(10));
        label.setFont(Font.font(50));

        pane.getChildren().add(label);
        pane.getChildren().add(getScrollPane());

        return pane;
    }

    public static ScrollPane scrollPane = new ScrollPane();
    public static GridPane gridPane = new GridPane();

    static private ScrollPane getScrollPane() {
        gridPane.setAlignment(Pos.TOP_CENTER);

        scrollPane.setContent(gridPane);
        scrollPane.fitToWidthProperty().set(true);
        gridPane.setBackground(new Background(new BackgroundFill(new Color(1, 1, 1, 1), new CornerRadii(0), Insets.EMPTY)));


        UIProgressBar progressBar = new UIProgressBar();
        progressBar.openNow("Loading levels...");
        new Thread(() -> {
            try {
                List<Level> levelList = DataManager.getLevels();
                progressBar.updateBar(0.5);
                for (int i = 0; i < levelList.size(); i++) {
                    Level level = levelList.get(i);

                    int finalI = i;
                    Platform.runLater(() -> gridPane.add(getLevelPane(level, finalI), 1, finalI));
                    double factor = 1f / levelList.size() * 0.5;

                    progressBar.updateBar(progressBar.getProgressBar().getProgress() + factor);
                }
                progressBar.updateBar(1);
                progressBar.close();
            } catch (Exception e) {
//                Runtime.getRuntime().exit(-1);
                throw new RuntimeException(e);
            }
        }).start();


        return scrollPane;
    }

    static private int selected = -1;
    static private Pane getLevelPane(Level level, int index) {
        GridPane pane = new GridPane(2, 2);
        Label title = new Label(level.getName());
        title.setFont(Font.font(30));
        Label creator = new Label(level.getCreator());
        creator.setFont(Font.font(20));
        pane.add(title, 1, 1);
        pane.add(creator, 1, 2);

        pane.setOnMouseClicked(mouseEvent -> {
            if (selected == index) {
                loadLevel(level);
            }
            selected = index;
            for (Node node : pane.getParent().getChildrenUnmodifiable()) {
                ((Pane) node).setBackground(new Background(new BackgroundFill(new Color(1, 1, 1, 1), new CornerRadii(0), Insets.EMPTY)));
            }
            pane.setBackground(new Background(new BackgroundFill(new Color(0.8, 0.8, 0.8, 0.5), new CornerRadii(0), Insets.EMPTY)));
        });

        return pane;
    }

    public static void loadLevel(Level level) {
        UIProgressBar progressBar = new UIProgressBar();
        progressBar.openNow("Loading level '" + level.getName() + "'...");
        progressBar.updateBar(0.5);
        log.info("Launched level {}", level.getName());
        new Thread(() -> {
            Utils.safeSleep(50);
            Platform.runLater(() -> {
                MainWindow.open(Main.getStage(), level);
                progressBar.updateBar(1);
                Platform.runLater(() -> {
                    Utils.safeSleep(500);
                    progressBar.close();
                });
            });
        }).start();
    }
}
