package xyz.tbvns.ui;

import javafx.application.Application;
import javafx.beans.InvalidationListener;
import javafx.scene.Node;
import javafx.scene.Scene;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.Pane;
import javafx.stage.Stage;
import lombok.Getter;
import lombok.Lombok;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import oshi.SystemInfo;
import xyz.tbvns.core.DataManager;
import xyz.tbvns.core.Decoder;
import xyz.tbvns.core.FilesLocations;
import xyz.tbvns.core.Objects.Level;
import xyz.tbvns.core.XmlUtils;
import xyz.tbvns.ui.LevelEditor.MainWindow;

import java.io.File;
import java.util.Arrays;

@Slf4j
public class Main extends Application {
    public static void main(String[] args) {
        launch(args);
    }


    @Getter private static Stage stage;
    @Override
    public void start(Stage stage) throws Exception {
        Main.stage = stage;

        log.info("Detected os: {}", new SystemInfo().getOperatingSystem().getFamily());
        log.info("GD Path for this os: {}", FilesLocations.getGeometryDashFolder());

        Pane levelSelect = LevelSelectScreen.pane();
        Scene scene = new Scene(levelSelect);
        setUpResizeSelectMenu(scene);

        stage.setScene(scene);
        stage.setTitle("PowerGD - Set of useful tools for creators");
        stage.setMinWidth(levelSelect.getMinWidth());
        stage.setMinHeight(levelSelect.getMinHeight());

        stage.show();
    }

    public static void setUpResizeSelectMenu(Scene scene) {
        scene.widthProperty().addListener((observableValue, number, t1) -> {
            LevelSelectScreen.label.setPrefWidth(t1.floatValue() - 20);
            LevelSelectScreen.scrollPane.setPrefWidth(t1.floatValue() - 20);
            LevelSelectScreen.gridPane.setPrefWidth(t1.floatValue() - 15);
            for (Node child : LevelSelectScreen.gridPane.getChildren()) {
                ((GridPane) child).setPrefWidth(t1.floatValue() - 15);
            }
        });

        scene.heightProperty().addListener((observableValue, number, t1) -> {
            LevelSelectScreen.scrollPane.setPrefHeight(
                    t1.floatValue() + 5 -
                            LevelSelectScreen.label.getHeight() -
                            LevelSelectScreen.label.getInsets().getTop() -
                            LevelSelectScreen.label.getInsets().getBottom()
            );
        });
    }

    public static void setUpResizeEditMenu(Scene scene) {
        scene.widthProperty().addListener((observableValue, number, t1) -> {
            MainWindow.menuBar.setPrefWidth(t1.floatValue());
            MainWindow.tabPane.setPrefWidth(t1.floatValue());
        });

        scene.heightProperty().addListener((observableValue, number, t1) -> {
            MainWindow.tabPane.setPrefHeight(t1.floatValue() - MainWindow.menuBar.getHeight());
        });
    }
}